#' Reduce the demand by percentage, where label==TRUE
#' @param 
#'		demand:			Vector of Energy Demand.
#'		label:			Boolean vector to label manipulated data
#'		percentage:		multiplied with demand to reduce energy
#' @examples
#'	data(demand)
#'	### Create Training Data
#'	data(demand)
#'	training_dimension1 <- demand[(1):(30*288),2]
#'	training_dimension2 <- demand[(1+1*288):(31*288),2]
#'	training_dimension3 <- demand[(1+2*288):(32*288),2]
#'	training_dimension4 <- demand[(1+3*288):(33*288),2]
#'	training_dimension5 <- demand[(1+4*288):(34*288),2]
#'	training <- c(training_dimension1,training_dimension2,training_dimension3,training_dimension4,training_dimension5)
#'	mean_forecast <- forecast(myDemand, 48, 5, 288, 800, f1_high_amplitude_points)
#'	### Our actual data is as follows:
#'	actual_dimension1 <- demand[(1+30*288):(31*288),2]
#'	actual_dimension2 <- demand[(1+31*288):(32*288),2]
#'	actual_dimension3 <- demand[(1+32*288):(33*288),2]
#'	actual_dimension4 <- demand[(1+33*288):(34*288),2]
#'	actual_dimension5 <- demand[(1+34*288):(35*288),2]
#'	### Now we manipulate day 5 (reduce demand to 0W at afternoon).
#'	label <- c(rep(FALSE,2*48), rep(TRUE, 1*48),rep(FALSE,3*48))
#'	actual_dimension5 <- stealElectricity(actual_dimension5, label, 0)
#'	actualData <- c(actual_dimension1,actual_dimension2,actual_dimension3,actual_dimension4,actual_dimension5)
#' 	windows <- getTimeWindows(actualData, 48) 							
#'  	features <- as.vector(colSums(sapply(windows,f1_high_amplitude_points, threshold=800)))	
#'  	prob <- toProbabilityMatrix(features,5)								
#'  	actual_entropy <- apply(prob,2, entropy)
#'	### Plot								
#'	plot(actual_entropy, type='l', ylim=c(0,1))
#'	lines(1:6,mean_forecast)
#'	points(3,actual_entropy[3], col='red', ylab='Entropy', xlab='Time', main='Anomalies')
stealElectricity <- function(demand, labels, percentage){ 		
	demand[labels] <- demand[labels] * percentage
	return(demand)
}

#' Derive features, compute entropy, perform a Holt Winters prediction of the next day
#' Note that, the length of demand must be be multiple times (windowSize * dimensionSize * measurementsPerDay)
#' @param 	
#'		dimensionSize:		Number of Rows. Note that, if length(features) cannot be divided by dimensionSize
#'					we fill the missing values with zero.
#' 		demand: 		Vector of Energy Demand.
#'		windowSize:		Amount of measurements in one window.
#'		feature_function:	function f1, f2 or f3 to derive the high-lvl feature
#'		threshold:		To divide according to the feature
forecast <- function(demand, windowSize, dimensionSize, measurementsPerDay, threshold, feature_function){
	windows <- getTimeWindows(demand, windowSize)
	features <- as.vector(colSums(sapply(windows,f1_high_amplitude_points, threshold=threshold)))
	prob <- toProbabilityMatrix(features,dimensionSize)
	ent <- apply(prob,2, entropy)								
	fit <- HoltWinters(ts(ent,frequency=measurementsPerDay/windowSize))						
	fore <- forecast::forecast(fit, h=measurementsPerDay/windowSize)	
	return(fore$mean)
}

#' Plot
#' @examples
#' Example:
#' data(demand)
#' myDemand_dimension1 <- demand[(1):(30*288),2]
#' myDemand_dimension2 <- demand[(1+1*288):(31*288),2]
#' myDemand_dimension3 <- demand[(1+2*288):(32*288),2]
#' myDemand_dimension4 <- demand[(1+3*288):(33*288),2]
#' myDemand_dimension5 <- demand[(1+4*288):(34*288),2]
#' myDemand <- c(myDemand_dimension1,myDemand_dimension2,myDemand_dimension3,myDemand_dimension4,myDemand_dimension5)
#' plot.forecast(myDemand, 48, 5, 288, 800, f1_high_amplitude_points)
plot.forecast <- function(demand, windowSize, dimensionSize, measurementsPerDay, threshold, feature_function){
	windows <- getTimeWindows(demand, windowSize)
	features <- as.vector(colSums(sapply(windows,f1_high_amplitude_points, threshold=threshold)))
	prob <- toProbabilityMatrix(features,dimensionSize)
	ent <- apply(prob,2, entropy)								
	fit <- HoltWinters(ts(ent,frequency=measurementsPerDay/windowSize))						
	fore <- forecast::forecast(fit, h=measurementsPerDay/windowSize)	

	par(mfrow=c(4,1),mar = c(5,6,4,2))
	plot(demand, type='l', main='Raw Data', ylab='Power (W)', xlab='Time', cex.axis=2, cex.lab=2, cex.main=2)
	plot(demand, col=feature_function(demand, threshold), main='Number of High Amplitude Points', ylab='Power (W)', xlab='Time', cex.axis=2, cex.lab=2, cex.main=2)
	abline(v=seq(1,length(demand),windowSize),lty=2)	
	barplot(prob, main='Distribution', xlab='Time', ylab='Distribution (%)')
	plot(fore)
}


#' Creates a matrix and converts the values to probabilities, so that each column equals 100 percent
#' @param 	features: 	input vector to split
#'		dimensionSize:	Number of Rows. Note that, if length(features) cannot be divided by dimensionSize
#'				we fill the missing values with zero.
#' @return 	matrix
#' @examples
#' Example 1: Convert three Houses (or three days) to a matrix whereas
#'	   each row is one house. 
#' house1 <- c(10,10,10 ,0,0 ,0 ,10,10,10,0 ,0 ,0)
#' house2 <- c(10,10,10,10,10,10,10,10,10,0 ,0 ,0)
#' house3 <- c(0 ,0 ,0 ,10,10,10,10,10,10,10,10,10)	
#' all <- c(house1,house2,house3)
#' windows <- getTimeWindows(all, 3)
#' features <- as.vector(colSums(sapply(windows,f1_high_amplitude_points, threshold=5)))
#' result <- toProbabilityMatrix(features,3)
#' barplot(result, xlab='Time', ylab='Distribution (%)')
#'
toProbabilityMatrix <- function(features, dimensionSize) {
	if(length(features) %% dimensionSize != 0) {
		remove <- (length(features) %% dimensionSize)
		features <- head(features,-remove)
		warning('Dimensions are not of equal size! Result may be wrong.')
	}
	features_matrix <- t(matrix(features , ncol=dimensionSize ) )
	result <- apply(features_matrix,2,function(x){x/sum(x)})
	return(result)
}

#' Splits the demand in time windows
#' @param 	demand: 	Vector of Energy Demand.
#'		windowSize:	Amount of measurements in one window.
#'				Note that, if demand cannot by divided by windowSize we remove additional values.
#' @return 	List of Vectors
getTimeWindows  <- function(demand, windowSize){ 
	if(length(demand) %% windowSize != 0) {
		fill <- abs((length(demand) %% windowSize)-windowSize)
		demand <- c(demand, rep(0,fill))
		warning('Measurements do not fit window size!')
	}
	result <- split(demand,as.numeric(gl(length(demand),windowSize,length(demand))))
	return(result)
}

#' Compute Shannon's entropy
#' Per default we normalize to a range [0,1] by dividing through log(length(p))
#' @param 	p: 		Vector of probabilities
#'		normalized:	If true, normalize to [0,1]
#' @return 	vector
#' @examples
#'  day1 <-  c(1,10,10,10,10,10,1,5,10,5,50,10,1,10,10,10,10,10,1,5,10,5,50,10)
#'  day2 <-  c(10,1,1,10,10,10,1,5,1,50,5,10,10,1,1,10,10,10,1,5,1,50,5,10)
#'  day3 <-  c(1,1,1,10,0,0,10,50,1,5,5,10,1,1,1,10,0,0,10,50,1,5,5,10)
#'  day4 <-  c(1,10,1,10,10,10,1,5,10,5,5,10,1,10,1,10,10,10,1,5,10,5,5,10)
#'  y <- c(day1,day2,day3,day4)									# We have 4 days each with 24 measurements
#'  windows <- getTimeWindows(y, 3) 								# We summarize three hours to one time window
#'  features <- as.vector(colSums(sapply(windows,f1_high_amplitude_points, threshold=5)))	
#'  prob <- toProbabilityMatrix(features,4)							# With a dimension size of 4 (days), we get 8 columns
#'  result <- apply(prob,2, entropy)								# Each column is an entropy value
#'  fit <- HoltWinters(ts(result,frequency=2))							# We assume, the pattern repeats every 2 time windows (6 hours)
#'  fore <- forecast(fit, h=2)									# We forecast the next 2 time windows (6 hours)
#'  plot(fore)
entropy <- function(p, normalized=TRUE){
	l <- length(p)

	if(normalized){
	#calculate normalized entropy with log n
		e_i <- (p * log(p)) / log(l)
	}
	else{
		e_i <- (p * log(p))
	}
	#Avoid NaN (generated by p=0)
	e_i[is.nan(e_i)] <- 0
	#Return entropy
	return(-sum(e_i))
}

#' F_1
#'
#' Computes the number of high amplitude points in a time window
#' @param 	demand: 	Vector of Energy Demand.
#'		threshold:	Amplitude to split high and low
#' @return 	boolean Vector
#' @examples 
#'
#' Example 1: Summarize points per time window
#' y <- c(1,1,1,10,10,10,1,5,1,5,5,10)
#' windows <- getTimeWindows(y, 3)
#' result <- as.vector(colSums(sapply(windows,f1_high_amplitude_points, threshold=5)))
#'
#' Example 2: Plot points
#' x <- 1:12
#' y <- c(1,1,1,10,10,10,1,5,1,5,5,10)
#' plot(x,y, type='l', main='Number of High Amplitude Points', ylab='Power (W)', xlab='Time',cex.axis=1.5, cex.lab=1.5, cex.main=1.5)
#' f1 <- f1_high_amplitude_points(y,5)
#' points(x[f1],y[f1], cex=2)
f1_high_amplitude_points <- function(demand, threshold){ 
	result <- ifelse(demand > threshold,TRUE,FALSE) 
	return(result)
}

#' F_2
#'
#' Computes the number of amplitude changes in a time window
#' @param 	demand: 	Vector of Energy Demand.
#'		threshold:	Amplitude to split similar and non-similar
#' @return 	boolean Vector
#' @examples  	see f1_high_amplitude_points
f2_amplitude_changes <- function(demand, threshold){ 
	result <- c(FALSE,ifelse(abs(diff(demand))<threshold,TRUE,FALSE)) 
	return(result)
}

#' F_3
#'
#' Computes the number of similar amplitudes in a row in a time window.
#' @param 	demand: 	Vector of Energy Demand.
#'		threshold:	Amplitude to split similar and non-similar	
#' @return 	boolean Vector
#' @examples  	see f1_high_amplitude_points 
f2_similar_amplitudes <- function(demand, threshold){ 
	result <- rep(ifelse(rle(threshold * round(demand/threshold))$lengths>3,TRUE,FALSE),rle(threshold * round(demand/threshold))$lengths)
	return(result)
}

